library IEEE;
use IEEE.std_logic_1164.all;

entity control_logic is
    port (
        clock: in STD_LOGIC;
        scounter: in STD_LOGIC_VECTOR(2 downto 0) ;
        instruction: in STD_LOGIC_VECTOR(7 downto 0);
        r0_bus: out STD_LOGIC;
        r1_bus: out STD_LOGIC;
        ld_out: out STD_LOGIC;
        lsl_r0: out STD_LOGIC;
        lsl_r1: out STD_LOGIC;
        swap_r0_r1: out STD_LOGIC;
        ld_r0: out STD_LOGIC;
        mem_read: out STD_LOGIC;
        mem_write: out STD_LOGIC;
        ld_r1: out STD_LOGIC;
        clr_r0: out STD_LOGIC;
        clr_r1: out STD_LOGIC;
        clr_temp: out STD_LOGIC;
        clr_out: out STD_LOGIC;
        ld_temp: out STD_LOGIC;
        clr_mbr: out STD_LOGIC;
        clr_mar1: out STD_LOGIC;
        ld_mar1: out STD_LOGIC;
        ld_mbr: out STD_LOGIC;
        clr_pc: out STD_LOGIC;
        clr_ir: out STD_LOGIC;
        ld_pc: out STD_LOGIC;
        ld_ir: out STD_LOGIC;
        clr_scntr: out STD_LOGIC;
        temp_bus: out STD_LOGIC;
        mem_bus: out STD_LOGIC
    );
end control_logic;

architecture control_logic_arch of control_logic is
begin
  process(instruction, clock, scounter)
	begin
  	if(clock'event and clock = '1') then
  		case instruction(7 downto 4) is
  			
  			-- R0 > OUT
  			when "0000" =>	if(scounter="000") then
  					r0_bus <= '1';
	  				ld_out <= '1';
  					clr_scntr <= '1';
 	 				end if;
  			
  			-- R1 > OUT
  			when "0001" =>	if(scounter="000") then
  					r1_bus <= '1';
	  				ld_out <= '1';
  					clr_scntr <= '1';
  					end if;
  				
  			-- RO < LSL R0
  			when "0010" =>	if(scounter="000") then
  					lsl_r0 <= '1';
	  				clr_scntr <= '1';
	  				end if;
  					
  			-- R1 < LSL R1
  			when "0011" =>	if(scounter="000") then
  					lsl_r1 <= '1';
  					clr_scntr <= '1';
  					end if;
  					
  			-- R0 > TMP ; R1 > R0 ; TMP > R1
  			when "0100" =>	if(scounter="000") then
  					swap_r0_r1 <= '1';
	  				r0_bus <= '1';
	  				ld_temp <= '1';
	  				r1_bus <= '1';
	  				ld_r0 <= '1';
	  				temp_bus <= '1';
	  				ld_r1 <= '1';
	  				clr_scntr <= '1';
	  				end if;
  			-- R0 < MEM
  			when "0101" =>	if(scounter="000") then
  					ld_r0 <= '1';
	  				mem_bus <= '1';
	  				mem_read <= '1';
	  				end if;
	  	
	  		-- R1 < MEM		
  			when "0110" =>	if(scounter="000") then
  					ld_r1 <= '1';
  					mem_bus <= '1';
  					mem_read <= '1';
  					end if;
  			
  			-- MEM < R0
  			when "0111" =>	if(scounter="000") then
  					r0_bus <= '1';
  					mem_write <= '1';
  					end if;
  					
  			-- MEM < R1
			when "1000" => if(scounter="000") then
  					r1_bus <= '1';
  					mem_write <= '1';
  					end if;
  			
  			when others => null;
  		end case;	
  	end if;
  	end process;					
end control_logic_arch;
